@@ -44,6 +44,7 @@ function _install_vpn_package {
 function _configure_vpn_ini_file {
     echo_summary "Configuring VPN ini file"
     local temp_ini=$(mktemp)
+    neutron_vpnaas_generate_config_files
     neutron_vpnaas_configure_agent $temp_ini
     sudo install -d -o $STACK_USER /etc/neutron/
     sudo install -m 644 -o $STACK_USER $temp_ini $Q_VPN_CONF_FILE
@@ -55,6 +56,9 @@ function configure_host_for_vpn_func_testing {
     if [ "$IS_GATE" == "True" ]; then
         configure_host_for_func_testing
     fi
+    # Note(pc_m): Need to ensure this is installed so we have
+    # oslo-config-generator present (as this script runs before tox.ini).
+    sudo pip install --force oslo.config
     _install_vpn_package
     _configure_vpn_ini_file
 }
